@@ -9,8 +9,8 @@
 # option) any later version.  See http://www.gnu.org/copyleft/gpl.html for
 # the full text of the license.
 
-import subprocess, tempfile, os.path, re, pwd, grp, os, time
-import fnmatch, glob, traceback, errno, sys, atexit, locale, imp
+import subprocess, tempfile, os.path, re, pwd, grp, os, time, io
+import fnmatch, glob, traceback, errno, sys, atexit, locale, imp, stat
 
 import xml.dom, xml.dom.minidom
 from xml.parsers.expat import ExpatError
@@ -19,16 +19,19 @@ if sys.version > '3':
     _python2 = False
     from urllib.error import URLError
     from urllib.request import urlopen
+    from urllib.parse import unquote
     (urlopen, URLError)  # pyflakes
 else:
     _python2 = True
     from urllib import urlopen
+    from urllib import unquote
     URLError = IOError
 
 import problem_report
 import apport
 import apport.fileutils
 from apport.packaging_impl import impl as packaging
+from apport.hookutils import kill_pkttyagent
 
 _data_dir = os.environ.get('APPORT_DATA_DIR', '/usr/share/apport')
 _hook_dir = '%s/package-hooks/' % (_data_dir)
@@ -64,27 +67,47 @@ def _transitive_dependencies(package, depends_set):
             _transitive_dependencies(d, depends_set)
 
 
-def _read_file(path):
+def _read_proc_link(path, pid=None, dir_fd=None):
+    '''Use readlink() to resolve link.
+
+    Return a string representing the path to which the symbolic link points.
+    '''
+    if not _python2 and dir_fd is not None:
+        return os.readlink(path, dir_fd=dir_fd)
+
+    return os.readlink("/proc/%s/%s" % (pid, path))
+
+
+def _read_proc_file(path, pid=None, dir_fd=None):
     '''Read file content.
 
     Return its content, or return a textual error if it failed.
     '''
     try:
-        with open(path, 'rb') as fd:
+        if not _python2 and dir_fd is not None:
+            proc_file = os.open(path, os.O_RDONLY | os.O_CLOEXEC, dir_fd=dir_fd)
+        else:
+            proc_file = "/proc/%s/%s" % (pid, path)
+
+        with io.open(proc_file, 'rb') as fd:
             return fd.read().strip().decode('UTF-8', errors='replace')
     except (OSError, IOError) as e:
         return 'Error: ' + str(e)
 
 
-def _read_maps(pid):
+def _read_maps(proc_pid_fd):
     '''Read /proc/pid/maps.
 
     Since /proc/$pid/maps may become unreadable unless we are ptracing the
     process, detect this, and attempt to attach/detach.
     '''
     maps = 'Error: unable to read /proc maps file'
+
+    if _python2:
+        return 'Error: python2 does not provide a secure way to read /proc maps file'
+
     try:
-        with open('/proc/%d/maps' % pid) as fd:
+        with open('maps', opener=lambda path, mode: os.open(path, mode, dir_fd=proc_pid_fd)) as fd:
             maps = fd.read().strip()
     except (OSError, IOError) as e:
         return 'Error: ' + str(e)
@@ -212,7 +235,7 @@ def _run_hook(report, ui, hook):
                 raise
     except StopIteration:
         return True
-    except Exception as e:
+    except Exception:
         hookname = os.path.splitext(os.path.basename(hook))[0].replace('-', '_')
         report['HookError_' + hookname] = traceback.format_exc()
         apport.error('hook %s crashed:', hook)
@@ -331,7 +354,7 @@ class Report(problem_report.ProblemReport):
             try:
                 self['SourcePackage'] = packaging.get_source(package)
             except ValueError:
-                # might not exist for non-free third-party packages
+                # might not exist for non-free third-party packages or snaps
                 pass
         if not version:
             return
@@ -356,6 +379,26 @@ class Report(problem_report.ProblemReport):
             self['Dependencies'] += '%s %s%s' % (
                 dep, v, self._customized_package_suffix(dep))
 
+    def add_snap_info(self, snap):
+        '''Add info about an installed Snap
+
+        This adds a Snap: field, containing name, version and channel.
+        It adds a SnapSource: field, if the snap has a Launchpad contact defined.
+        '''
+        self['Snap'] = '%s %s (%s)' % (snap.get("name"), snap.get("version"),
+                                       snap.get("channel", "unknown"))
+        # Automatically handle snaps which have a Launchpad contact defined
+        if snap.get("contact"):
+            # Parse Launchpad project (e.g. 'subiquity') or source package string
+            # (e.g. 'ubuntu/+source/gnome-calculator') from snap 'contact'
+            # Additionaly, extract any tag/tags defined in the contact URL.
+            p = r'^https?:\/\/.*launchpad\.net\/((?:[^\/]+\/\+source\/)?[^\/]+)(?:.*field\.tags?=([^&]+))?'
+            m = re.search(p, unquote(snap.get("contact", "")))
+            if m and m.group(1):
+                self['SnapSource'] = m.group(1)
+                if m.group(2):
+                    self['SnapTags'] = m.group(2)
+
     def add_os_info(self):
         '''Add operating system information.
 
@@ -383,7 +426,12 @@ class Report(problem_report.ProblemReport):
         groups = [name for name, p, gid, memb in grp.getgrall()
                   if user in memb and gid < 1000]
         groups.sort()
-        self['UserGroups'] = ' '.join(groups)
+        if groups:
+            self['UserGroups'] = ' '.join(groups)
+        else:
+            # the docs indicate this is optional but a lot of tests expect
+            # UserGroups to exist
+            self['UserGroups'] = 'N/A'
 
     def _check_interpreted(self):
         '''Check if process is a script.
@@ -417,7 +465,7 @@ class Report(problem_report.ProblemReport):
             return
 
         cmdargs = self['ProcCmdline'].split('\0')
-        bindirs = ['/bin/', '/sbin/', '/usr/bin/', '/usr/sbin/']
+        bindirs = ['/usr/bin/', '/usr/sbin/', '/bin/', '/sbin/']
 
         # filter out interpreter options
         while len(cmdargs) >= 2 and cmdargs[1].startswith('-'):
@@ -505,13 +553,13 @@ class Report(problem_report.ProblemReport):
             pathlist = [path]
 
             if not module and desc[2] == imp.PKG_DIRECTORY:
-                    module = ['__init__']
+                module = ['__init__']
 
         if path and path.endswith('.pyc'):
             path = path[:-1]
         return path
 
-    def add_proc_info(self, pid=None, extraenv=[]):
+    def add_proc_info(self, pid=None, proc_pid_fd=None, extraenv=[]):
         '''Add /proc/pid information.
 
         If neither pid nor self.pid are given, it defaults to the process'
@@ -535,27 +583,41 @@ class Report(problem_report.ProblemReport):
         - _LogindSession: logind cgroup path, if present (Used for filtering
           out crashes that happened in a session that is not running any more)
         '''
-        if not pid:
-            pid = self.pid or os.getpid()
-        if not self.pid:
-            self.pid = int(pid)
-        pid = str(pid)
+        if not proc_pid_fd:
+            if not pid:
+                pid = self.pid or os.getpid()
+            if not self.pid:
+                self.pid = int(pid)
+            pid = str(pid)
+            if not _python2:
+                try:
+                    proc_pid_fd = os.open('/proc/%s' % pid, os.O_RDONLY | os.O_PATH | os.O_DIRECTORY)
+                except OSError as e:
+                    if e.errno in (errno.EPERM, errno.EACCES):
+                        raise ValueError('not accessible')
+                    if e.errno == errno.ENOENT:
+                        raise ValueError('invalid process')
+                    else:
+                        raise
 
         try:
-            self['ProcCwd'] = os.readlink('/proc/' + pid + '/cwd')
+            self['ProcCwd'] = _read_proc_link('cwd', pid, proc_pid_fd)
         except OSError:
             pass
-        self.add_proc_environ(pid, extraenv)
-        self['ProcStatus'] = _read_file('/proc/' + pid + '/status')
-        self['ProcCmdline'] = _read_file('/proc/' + pid + '/cmdline').rstrip('\0')
-        self['ProcMaps'] = _read_maps(int(pid))
-        try:
-            self['ExecutablePath'] = os.readlink('/proc/' + pid + '/exe')
-        except OSError as e:
-            if e.errno == errno.ENOENT:
-                raise ValueError('invalid process')
-            else:
-                raise
+        self.add_proc_environ(pid=pid, proc_pid_fd=proc_pid_fd, extraenv=extraenv)
+        self['ProcStatus'] = _read_proc_file('status', pid, proc_pid_fd)
+        self['ProcCmdline'] = _read_proc_file('cmdline', pid, proc_pid_fd).rstrip('\0')
+        self['ProcMaps'] = _read_maps(proc_pid_fd)
+        if 'ExecutablePath' not in self:
+            try:
+                self['ExecutablePath'] = _read_proc_link('exe', pid, proc_pid_fd)
+            except OSError as e:
+                if e.errno in (errno.EPERM, errno.EACCES):
+                    raise ValueError('not accessible')
+                if e.errno == errno.ENOENT:
+                    raise ValueError('invalid process')
+                else:
+                    raise
         for p in ('rofs', 'rwfs', 'squashmnt', 'persistmnt'):
             if self['ExecutablePath'].startswith('/%s/' % p):
                 self['ExecutablePath'] = self['ExecutablePath'][len('/%s' % p):]
@@ -577,18 +639,17 @@ class Report(problem_report.ProblemReport):
             # On Linux 2.6.28+, 'current' is world readable, but read() gives
             # EPERM; Python 2.5.3+ crashes on that (LP: #314065)
             if os.getuid() == 0:
-                with open('/proc/' + pid + '/attr/current') as fd:
-                    val = fd.read().strip()
+                val = _read_proc_file('attr/current', pid, proc_pid_fd)
                 if val != 'unconfined':
                     self['ProcAttrCurrent'] = val
         except (IOError, OSError):
             pass
 
-        ret = self.get_logind_session(pid)
+        ret = self.get_logind_session(pid, proc_pid_fd)
         if ret:
             self['_LogindSession'] = ret[0]
 
-    def add_proc_environ(self, pid=None, extraenv=[]):
+    def add_proc_environ(self, pid=None, extraenv=[], proc_pid_fd=None):
         '''Add environment information.
 
         If pid is not given, it defaults to the process' current pid.
@@ -605,12 +666,15 @@ class Report(problem_report.ProblemReport):
                      'LC_TELEPHONE', 'LC_MEASUREMENT', 'LC_IDENTIFICATION',
                      'LOCPATH'] + extraenv
 
-        if not pid:
-            pid = os.getpid()
-        pid = str(pid)
+        if not proc_pid_fd:
+            if not pid:
+                pid = os.getpid()
+            pid = str(pid)
+            if not _python2:
+                proc_pid_fd = os.open('/proc/%s' % pid, os.O_RDONLY | os.O_PATH | os.O_DIRECTORY)
 
         self['ProcEnviron'] = ''
-        env = _read_file('/proc/' + pid + '/environ').replace('\n', '\\n')
+        env = _read_proc_file('environ', pid, proc_pid_fd).replace('\n', '\\n')
         if env.startswith('Error:'):
             self['ProcEnviron'] = env
         else:
@@ -705,7 +769,8 @@ class Report(problem_report.ProblemReport):
         files there.
 
         Raises a IOError if the core dump is invalid/truncated, or OSError if
-        calling gdb fails.
+        calling gdb fails, or FileNotFoundError if gdb or the crashing
+        executable cannot be found.
         '''
         if 'CoreDump' not in self or 'ExecutablePath' not in self:
             return
@@ -718,6 +783,9 @@ class Report(problem_report.ProblemReport):
                        'GLibAssertionMessage': 'print __glib_assert_msg',
                        'NihAssertionMessage': 'print (char*) __nih_abort_msg'}
         gdb_cmd, environ = self.gdb_command(rootdir, gdb_sandbox)
+        if not gdb_cmd:
+            raise FileNotFoundError(errno.ENOENT, os.strerror(errno.ENOENT),
+                                    'gdb not found in retracing env')
 
         # limit maximum backtrace depth (to avoid looped stacks)
         gdb_cmd += ['--batch', '--ex', 'set backtrace limit 2000']
@@ -727,18 +795,29 @@ class Report(problem_report.ProblemReport):
         for name, cmd in gdb_reports.items():
             value_keys.append(name)
             gdb_cmd += ['--ex', 'p -99', '--ex', cmd]
+        # End with our separator, ensures gdb's return code is as expected
+        value_keys.append('separator')
+        gdb_cmd += ['--ex', 'p -99']
 
         out = _command_output(gdb_cmd, env=environ).decode('UTF-8', errors='replace')
 
         # check for truncated stack trace
-        if 'is truncated: expected core file size' in out:
+        if 'is truncated: expected core file size' in out or \
+                'is not a core dump' in out:
             if 'warning:' in out:
-                warnings = '\n'.join([l for l in out.splitlines() if 'warning:' in l])
+                warnings = '\n'.join([line for line in out.splitlines() if
+                                      'warning:' in line])
             elif 'Warning:' in out:
-                warnings = '\n'.join([l for l in out.splitlines() if 'Warning:' in l])
+                warnings = '\n'.join([line for line in out.splitlines() if
+                                      'Warning:' in line])
+            else:
+                warnings = out.splitlines()[0]
             reason = 'Invalid core dump: ' + warnings.strip()
             self['UnreportableReason'] = reason
             raise IOError(reason)
+        elif out.split('\n')[0].endswith('No such file or directory.'):
+            raise FileNotFoundError(errno.ENOENT, os.strerror(errno.ENOENT),
+                                    'executable file for coredump not found')
 
         # split the output into the various fields
         part_re = re.compile(r'^\$\d+\s*=\s*-99$', re.MULTILINE)
@@ -852,6 +931,11 @@ class Report(problem_report.ProblemReport):
         return True if the hook requested to stop the report filing process,
         False otherwise.
         '''
+        ret = self._add_hooks_info(ui, package, srcpackage)
+        kill_pkttyagent()
+        return ret
+
+    def _add_hooks_info(self, ui, package, srcpackage):
         # determine package names, unless already given as arguments
         # avoid path traversal
         if not package:
@@ -966,25 +1050,39 @@ class Report(problem_report.ProblemReport):
         ifpath = os.path.expanduser(_ignore_file)
         if orig_home is not None:
             os.environ['HOME'] = orig_home
+        contents = ''
+        fd = None
+        f = None
+        egid = os.getegid()
         euid = os.geteuid()
         try:
             # drop permissions temporarily to try open users ignore file
+            os.setegid(os.getgid())
             os.seteuid(os.getuid())
-            fp = open(ifpath, 'r')
+            fd = os.open(ifpath, os.O_NOFOLLOW | os.O_RDONLY)
+            st = os.fstat(fd)
+            if stat.S_ISREG(st.st_mode):
+                f = os.fdopen(fd, 'r')
+                # Limit size to prevent DoS
+                contents = f.read(50000)
         except (IOError, OSError):
-            fp = None
+            pass
         finally:
+            if f is not None:
+                f.close()
+            elif fd is not None:
+                os.close(fd)
             os.seteuid(euid)
-        if fp is None or os.fstat(fp.fileno()).st_size == 0:
+            os.setegid(egid)
+
+        if contents == '':
             # create a document from scratch
             dom = xml.dom.getDOMImplementation().createDocument(None, 'apport', None)
         else:
             try:
-                dom = xml.dom.minidom.parse(fp)
+                dom = xml.dom.minidom.parseString(contents)
             except ExpatError as e:
                 raise ValueError('%s has invalid format: %s' % (_ignore_file, str(e)))
-        if fp is not None:
-            fp.close()
         # remove whitespace so that writing back the XML does not accumulate
         # whitespace
         dom.documentElement.normalize()
@@ -1050,10 +1148,13 @@ class Report(problem_report.ProblemReport):
             return False
 
         # search for existing entry and update it
-        for ignore in dom.getElementsByTagName('ignore'):
-            if ignore.getAttribute('program') == self['ExecutablePath']:
-                if float(ignore.getAttribute('mtime')) >= cur_mtime:
-                    return True
+        try:
+            for ignore in dom.getElementsByTagName('ignore'):
+                if ignore.getAttribute('program') == self['ExecutablePath']:
+                    if float(ignore.getAttribute('mtime')) >= cur_mtime:
+                        return True
+        except (ValueError, KeyError):
+            pass
 
         return False
 
@@ -1445,6 +1546,8 @@ class Report(problem_report.ProblemReport):
         '''
         if 'ProcMaps' not in self or 'Stacktrace' not in self or 'Signal' not in self:
             return None
+        if 'Errno 13' in self['ProcMaps']:
+            return None
 
         stack = []
         failed = 0
@@ -1551,8 +1654,7 @@ class Report(problem_report.ProblemReport):
                            if gdb_sandbox else None)
         gdb_path = _which_extrapath('gdb', gdb_sandbox_bin)
         if not gdb_path:
-            apport.fatal('gdb does not exist in the %ssandbox nor on the host'
-                         % ('gdb ' if not same_arch else ''))
+            return '', ''
         command = [gdb_path]
         environ = None
 
@@ -1576,8 +1678,8 @@ class Report(problem_report.ProblemReport):
                         '--ex', 'add-auto-load-safe-path ' + sandbox,
                         # needed to fix /lib64/ld-linux-x86-64.so.2 broken
                         # symlink
-                        '--ex', 'set solib-search-path %s/lib/%s' % \
-                        (sandbox, native_multiarch)]
+                        '--ex', 'set solib-search-path %s/lib/%s:%s/usr/lib/%s' % \
+                        (sandbox, native_multiarch, sandbox, native_multiarch)]
             if gdb_sandbox:
                 ld_lib_path = '%s/lib:%s/lib/%s:%s/usr/lib/%s:%s/usr/lib' % \
                     (gdb_sandbox, gdb_sandbox, native_multiarch,
@@ -1594,7 +1696,6 @@ class Report(problem_report.ProblemReport):
                             gdb_sandbox]
             executable = sandbox + '/' + executable
 
-        assert os.path.exists(executable)
         command += ['--ex', 'file "%s"' % executable]
 
         if 'CoreDump' in self:
@@ -1672,15 +1773,20 @@ class Report(problem_report.ProblemReport):
                                           int(m.group(2), 16), m.group(3)))
 
     @classmethod
-    def get_logind_session(klass, pid):
+    def get_logind_session(klass, pid=None, proc_pid_fd=None):
         '''Get logind session path and start time.
 
-        Return (session_id, session_start_timestamp) if pid is in a logind
+        Return (session_id, session_start_timestamp) if process is in a logind
         session, or None otherwise.
         '''
+        if not _python2 and proc_pid_fd is not None:
+            cgroup_file = os.open('cgroup', os.O_RDONLY, dir_fd=proc_pid_fd)
+        else:
+            cgroup_file = "/proc/%s/cgroup" % pid
+
         # determine cgroup
         try:
-            with open('/proc/%s/cgroup' % pid) as f:
+            with io.open(cgroup_file) as f:
                 for line in f:
                     line = line.strip()
                     if 'name=systemd:' in line and line.endswith('.scope') and '/session-' in line:
