@@ -14,7 +14,7 @@ implementation (like GTK, Qt, or CLI).
 # the full text of the license.
 
 import glob, sys, os.path, optparse, traceback, locale, gettext, re, io
-import errno, zlib
+import errno, zlib, gzip
 import subprocess, threading, webbrowser
 import signal
 import time
@@ -33,7 +33,7 @@ else:
     from configparser import ConfigParser
     PY3 = True
 
-__version__ = '2.20.9'
+__version__ = '2.20.11'
 
 
 def excstr(exception):
@@ -112,7 +112,7 @@ def thread_collect_info(report, reportfile, package, ui, symptom_script=None,
             report['Symptom'] = os.path.splitext(os.path.basename(symptom_script))[0]
         except StopIteration:
             sys.exit(0)
-        except Exception as e:
+        except Exception:
             apport.error('symptom script %s crashed:', symptom_script)
             traceback.print_exc()
             sys.exit(0)
@@ -259,6 +259,9 @@ class UserInterface:
             # and the session started before the report's "Date".
             if logind_session and '_LogindSession' in self.report and \
                'Date' in self.report:
+                # report.get_timestamp() can return None
+                if not self.report.get_timestamp():
+                    continue
                 if logind_session[0] != self.report['_LogindSession'] or \
                    logind_session[1] > self.report.get_timestamp():
                     continue
@@ -395,7 +398,17 @@ class UserInterface:
         mark the report for uploading.
         '''
         self.report = apport.Report('Hang')
-        self.report.add_proc_info(pid)
+        try:
+            self.report.add_proc_info(pid)
+        except ValueError as e:
+            if str(e) == 'invalid process':
+                self.ui_error_message(_('Invalid PID'),
+                                      _('The specified process ID does not exist.'))
+                sys.exit(1)
+            elif str(e) == 'not accessible':
+                self.ui_error_message(_('Not your PID'),
+                                      _('The specified process ID does not belong to you.'))
+                sys.exit(1)
         self.report.add_package_info()
         path = self.report.get('ExecutablePath', '')
         self.cur_package = apport.fileutils.find_file_package(path)
@@ -522,8 +535,13 @@ class UserInterface:
 
         if self.options.save:
             try:
-                with open(os.path.expanduser(self.options.save), 'wb') as f:
-                    self.report.write(f)
+                savefile = os.path.expanduser(self.options.save)
+                if savefile.endswith(".gz"):
+                    with gzip.open(savefile, 'wb') as f:
+                        self.report.write(f)
+                else:
+                    with open(os.path.expanduser(self.options.save), 'wb') as f:
+                        self.report.write(f)
             except (IOError, OSError) as e:
                 self.ui_error_message(_('Cannot create report'), excstr(e))
         else:
@@ -703,23 +721,23 @@ class UserInterface:
                 self.ui_error_message(_('Invalid problem report'), excstr(e))
             return True
         elif self.options.window:
-                self.ui_info_message('', _('After closing this message '
-                                           'please click on an application window to report a problem about it.'))
-                xprop = subprocess.Popen(['xprop', '_NET_WM_PID'],
-                                         stdout=subprocess.PIPE, stderr=subprocess.PIPE)
-                (out, err) = xprop.communicate()
-                if xprop.returncode == 0:
-                    try:
-                        self.options.pid = int(out.split()[-1])
-                    except ValueError:
-                        self.ui_error_message(_('Cannot create report'),
-                                              _('xprop failed to determine process ID of the window'))
-                        return True
-                    return self.run_report_bug()
-                else:
+            self.ui_info_message('', _('After closing this message '
+                                       'please click on an application window to report a problem about it.'))
+            xprop = subprocess.Popen(['xprop', '_NET_WM_PID'],
+                                     stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+            (out, err) = xprop.communicate()
+            if xprop.returncode == 0:
+                try:
+                    self.options.pid = int(out.split()[-1])
+                except ValueError:
                     self.ui_error_message(_('Cannot create report'),
-                                          _('xprop failed to determine process ID of the window') + '\n\n' + err)
+                                          _('xprop failed to determine process ID of the window'))
                     return True
+                return self.run_report_bug()
+            else:
+                self.ui_error_message(_('Cannot create report'),
+                                      _('xprop failed to determine process ID of the window') + '\n\n' + err)
+                return True
         else:
             return self.run_crashes()
 
@@ -1093,7 +1111,7 @@ class UserInterface:
                                                     args=(self.report, self.report_file, self.cur_package,
                                                           hookui, symptom_script, ignore_uninstalled))
                 icthread.start()
-                while icthread.isAlive():
+                while icthread.is_alive():
                     self.ui_pulse_info_collection_progress()
                     if hookui:
                         try:
@@ -1158,7 +1176,7 @@ class UserInterface:
                 bpthread = apport.REThread.REThread(target=self.report.search_bug_patterns,
                                                     args=(self.crashdb.get_bugpattern_baseurl(),))
                 bpthread.start()
-                while bpthread.isAlive():
+                while bpthread.is_alive():
                     self.ui_pulse_info_collection_progress()
                     try:
                         bpthread.join(0.1)
@@ -1179,7 +1197,7 @@ class UserInterface:
                 known_thread = apport.REThread.REThread(target=self.crashdb.known,
                                                         args=(self.report,))
                 known_thread.start()
-                while known_thread.isAlive():
+                while known_thread.is_alive():
                     self.ui_pulse_info_collection_progress()
                     try:
                         known_thread.join(0.1)
@@ -1197,7 +1215,7 @@ class UserInterface:
             # might damage the stack trace
             anonymize_thread = apport.REThread.REThread(target=self.report.anonymize)
             anonymize_thread.start()
-            while anonymize_thread.isAlive():
+            while anonymize_thread.is_alive():
                 self.ui_pulse_info_collection_progress()
                 try:
                     anonymize_thread.join(0.1)
@@ -1276,7 +1294,7 @@ class UserInterface:
         try:
             try:
                 subprocess.call(sudo_prefix + ['xdg-open', url])
-            except OSError as e:
+            except OSError:
                 # fall back to webbrowser
                 webbrowser.open(url, new=True, autoraise=True)
                 sys.exit(0)
@@ -1324,7 +1342,7 @@ class UserInterface:
         upthread = apport.REThread.REThread(target=self.crashdb.upload,
                                             args=(self.report, progress_callback))
         upthread.start()
-        while upthread.isAlive():
+        while upthread.is_alive():
             self.ui_set_upload_progress(__upload_progress)
             try:
                 upthread.join(0.1)
