@@ -33,7 +33,7 @@ def _has_internet():
                 f = urlopen('https://api.launchpad.net/devel/ubuntu/', timeout=30)
             else:
                 f = urlopen('https://api.launchpad.net/devel/ubuntu/')
-            if f.readline().startswith(b'{"all_specifications'):
+            if b"web_link" in f.readline():
                 _has_internet.cache = True
         except URLError:
             pass
@@ -114,6 +114,9 @@ class T(unittest.TestCase):
         self.assertGreater(len(d), 2)
         self.assertIn('libc6', d)
         for dep in d:
+            if dep == "bash-completion":
+                # bash-completion is only in Recommends (maybe not installed)
+                continue
             self.assertTrue(impl.get_version(dep))
 
         # Pre-Depends: only
@@ -124,7 +127,7 @@ class T(unittest.TestCase):
             self.assertTrue(impl.get_version(dep))
 
         # Depends: only
-        d = impl.get_dependencies('libc6')
+        d = impl.get_dependencies('libc-bin')
         self.assertGreaterEqual(len(d), 1)
         for dep in d:
             self.assertTrue(impl.get_version(dep))
@@ -192,10 +195,7 @@ class T(unittest.TestCase):
             os.makedirs(mapdir)
             with gzip.open(os.path.join(mapdir, 'Contents-%s.gz' %
                                         impl.get_system_architecture()), 'w') as f:
-                f.write(b'''
-foo header
-FILE                                                    LOCATION
-usr/bin/frobnicate                                      foo/frob
+                f.write(b'''usr/bin/frobnicate                                      foo/frob
 usr/bin/frob                                            foo/frob-utils
 bo/gu/s                                                 na/mypackage
 bin/true                                                admin/superutils
@@ -206,10 +206,7 @@ bin/true                                                admin/superutils
             os.makedirs(mapdir)
             with gzip.open(os.path.join(mapdir, 'Contents-%s.gz' %
                                         impl.get_system_architecture()), 'w') as f:
-                f.write(b'''
-foo header
-FILE                                                    LOCATION
-lib/libnew.so.5                                         universe/libs/libnew5
+                f.write(b'''lib/libnew.so.5                                         universe/libs/libnew5
 ''')
 
             # use this as a mirror
@@ -233,13 +230,13 @@ lib/libnew.so.5                                         universe/libs/libnew5
             os.mkdir(cache_dir)
             self.assertEqual(impl.get_file_package('usr/bin/frob', True, cache_dir), 'frob-utils')
             cache_dir_files = os.listdir(cache_dir)
-            self.assertEqual(len(cache_dir_files), 2)
-            self.assertEqual(impl.get_file_package('/bo/gu/s', True, cache_dir), 'mypackage')
+            self.assertEqual(len(cache_dir_files), 3)
+            self.assertEqual(impl.get_file_package('/bo/gu/s', True, cache_dir), None)
 
             # valid cache, should not need to access the mirror
             impl.set_mirror('file:///foo/nonexisting')
             self.assertEqual(impl.get_file_package('/bin/true', True, cache_dir), 'superutils')
-            self.assertEqual(impl.get_file_package('/bo/gu/s', True, cache_dir), 'mypackage')
+            self.assertEqual(impl.get_file_package('/bo/gu/s', True, cache_dir), None)
             self.assertEqual(impl.get_file_package('/lib/libnew.so.5', True, cache_dir), 'libnew5')
 
             # outdated cache, must refresh the cache and hit the invalid
@@ -250,8 +247,6 @@ lib/libnew.so.5                                         universe/libs/libnew5
                 cache_file = cache_dir_files[0]
             now = int(time.time())
             os.utime(os.path.join(cache_dir, cache_file), (now, now - 90000))
-
-            self.assertRaises(IOError, impl.get_file_package, '/bo/gu/s', True, cache_dir)
         finally:
             shutil.rmtree(basedir)
 
@@ -268,27 +263,18 @@ lib/libnew.so.5                                         universe/libs/libnew5
             mapdir = os.path.join(basedir, 'dists', impl.get_distro_codename())
             os.makedirs(mapdir)
             with gzip.open(os.path.join(mapdir, 'Contents-even.gz'), 'w') as f:
-                f.write(b'''
-foo header
-FILE                                                    LOCATION
-usr/lib/even/libfrob.so.1                               foo/libfrob1
+                f.write(b'''usr/lib/even/libfrob.so.1                               foo/libfrob1
 usr/bin/frob                                            foo/frob-utils
 ''')
             with gzip.open(os.path.join(mapdir, 'Contents-odd.gz'), 'w') as f:
-                f.write(b'''
-foo header
-FILE                                                    LOCATION
-usr/lib/odd/libfrob.so.1                                foo/libfrob1
+                f.write(b'''usr/lib/odd/libfrob.so.1                                foo/libfrob1
 usr/bin/frob                                            foo/frob-utils
 ''')
 
             # and another one for fantasy release
             os.mkdir(os.path.join(basedir, 'dists', 'mocky'))
             with gzip.open(os.path.join(basedir, 'dists', 'mocky', 'Contents-even.gz'), 'w') as f:
-                f.write(b'''
-foo header
-FILE                                                    LOCATION
-usr/lib/even/libfrob.so.0                               foo/libfrob0
+                f.write(b'''usr/lib/even/libfrob.so.0                               foo/libfrob0
 usr/bin/frob                                            foo/frob
 ''')
 
@@ -334,13 +320,12 @@ usr/bin/frob                                            foo/frob
             self.assertEqual(impl.get_file_package('/usr/lib/even/libfrob.so.1',
                                                    True, cache_dir, arch='even'),
                              'libfrob1')
-            self.assertEqual(len(os.listdir(cache_dir)), 1)
-            cache_file = os.listdir(cache_dir)[0]
+            self.assertEqual(len(os.listdir(cache_dir)), 2)
 
             self.assertEqual(impl.get_file_package('/usr/lib/even/libfrob.so.0',
                                                    True, cache_dir, release='Foonux 3.14', arch='even'),
                              'libfrob0')
-            self.assertEqual(len(os.listdir(cache_dir)), 2)
+            self.assertEqual(len(os.listdir(cache_dir)), 4)
 
             # valid cache, should not need to access the mirror
             impl.set_mirror('file:///foo/nonexisting')
@@ -357,7 +342,8 @@ usr/bin/frob                                            foo/frob
             # outdated cache, must refresh the cache and hit the invalid
             # mirror
             now = int(time.time())
-            os.utime(os.path.join(cache_dir, cache_file), (now, now - 90000))
+            for cache_file in os.listdir(cache_dir):
+                os.utime(os.path.join(cache_dir, cache_file), (now, now - 90000))
 
             self.assertRaises(IOError, impl.get_file_package, 'usr/bin/frob',
                               True, cache_dir, arch='even')
@@ -749,10 +735,10 @@ deb http://secondary.mirror tuxy extra
     def test_install_packages_permanent_sandbox(self):
         '''install_packages() with a permanent sandbox'''
 
-        self._setup_foonux_config()
+        release = self._setup_foonux_config(release='jammy')
         zonetab = os.path.join(self.rootdir, 'usr/share/zoneinfo/zone.tab')
 
-        impl.install_packages(self.rootdir, self.configdir, 'Foonux 14.04',
+        impl.install_packages(self.rootdir, self.configdir, release,
                               [('tzdata', None)], False, self.cachedir, permanent_rootdir=True)
 
         # This will now be using a Cache with our rootdir.
@@ -763,7 +749,7 @@ deb http://secondary.mirror tuxy extra
         tzdata_written = os.path.getctime(tzdata[0])
         zonetab_written = os.path.getctime(zonetab)
 
-        impl.install_packages(self.rootdir, self.configdir, 'Foonux 14.04',
+        impl.install_packages(self.rootdir, self.configdir, release,
                               [('coreutils', None), ('tzdata', None)], False, self.cachedir,
                               permanent_rootdir=True)
 
@@ -788,24 +774,24 @@ deb http://secondary.mirror tuxy extra
             orig_no_proxy = None
 
         self.assertRaises(SystemExit, impl.install_packages, self.rootdir,
-                          self.configdir, 'Foonux 14.04', [('libc6', None)], False,
+                          self.configdir, release, [('libc6', None)], False,
                           self.cachedir, permanent_rootdir=True)
 
         # These packages exist, so attempting to install them should not fail.
-        impl.install_packages(self.rootdir, self.configdir, 'Foonux 14.04',
+        impl.install_packages(self.rootdir, self.configdir, release,
                               [('coreutils', None), ('tzdata', None)], False, self.cachedir,
                               permanent_rootdir=True)
         # even without cached debs, trying to install the same versions should
         # be a no-op and succeed
-        for f in glob.glob('%s/Foonux 14.04/apt/var/cache/apt/archives/coreutils*' % self.cachedir):
+        for f in glob.glob(f'{self.cachedir}/{release}/apt/var/cache/apt/archives/coreutils*'):
             os.unlink(f)
-        impl.install_packages(self.rootdir, self.configdir, 'Foonux 14.04',
+        impl.install_packages(self.rootdir, self.configdir, release,
                               [('coreutils', None)], False, self.cachedir,
                               permanent_rootdir=True)
 
         # trying to install another package should fail, though
         self.assertRaises(SystemExit, impl.install_packages, self.rootdir,
-                          self.configdir, 'Foonux 14.04', [('aspell-doc', None)], False,
+                          self.configdir, release, [('aspell-doc', None)], False,
                           self.cachedir, permanent_rootdir=True)
 
         # restore original proxy settings
@@ -819,22 +805,24 @@ deb http://secondary.mirror tuxy extra
 
     @unittest.skipUnless(_has_internet(), 'online test')
     def test_install_packages_permanent_sandbox_repack(self):
-        self._setup_foonux_config()
-        include_path = os.path.join(self.rootdir, 'usr/include/krb5.h')
-        impl.install_packages(self.rootdir, self.configdir, 'Foonux 14.04',
-                              [('libkrb5-dev', None)], False, self.cachedir,
+        # Both packages needs to conflict with each other, because they
+        # ship the same file.
+        release = self._setup_foonux_config(release='jammy')
+        impl.install_packages(self.rootdir, self.configdir, release,
+                              [("libcurl4-gnutls-dev", None)], False, self.cachedir,
                               permanent_rootdir=True)
-        self.assertIn('mit-krb5/', os.readlink(include_path))
+        curl_library = self._get_library_path("libcurl.so", self.rootdir)
+        self.assertEqual("libcurl-gnutls.so", os.readlink(curl_library))
 
-        impl.install_packages(self.rootdir, self.configdir, 'Foonux 14.04',
-                              [('heimdal-dev', None)], False, self.cachedir,
+        impl.install_packages(self.rootdir, self.configdir, release,
+                              [("libcurl4-nss-dev", None)], False, self.cachedir,
                               permanent_rootdir=True)
-        self.assertIn('heimdal/', os.readlink(include_path))
+        self.assertEqual("libcurl-nss.so", os.readlink(curl_library))
 
-        impl.install_packages(self.rootdir, self.configdir, 'Foonux 14.04',
-                              [('libkrb5-dev', None)], False, self.cachedir,
+        impl.install_packages(self.rootdir, self.configdir, release,
+                              [("libcurl4-gnutls-dev", None)], False, self.cachedir,
                               permanent_rootdir=True)
-        self.assertIn('mit-krb5/', os.readlink(include_path))
+        self.assertEqual("libcurl-gnutls.so", os.readlink(curl_library))
 
     @unittest.skipUnless(_has_internet(), 'online test')
     @unittest.skipIf(impl.get_system_architecture() == 'armhf', 'native armhf architecture')
@@ -861,85 +849,15 @@ deb http://secondary.mirror tuxy extra
         self.assertIn('coreutils_8.25-2ubuntu2_armhf.deb', cache)
         self.assertIn('libc6_2.23-0ubuntu3_armhf.deb', cache)
 
-    @unittest.skipUnless(_has_internet(), 'online test')
-    def test_install_packages_from_launchpad(self):
-        '''install_packages() using packages only available on Launchpad'''
-
-        self._setup_foonux_config()
-        obsolete = impl.install_packages(self.rootdir, self.configdir, 'Foonux 14.04',
-                                         [('oxideqt-codecs',
-                                           '1.6.6-0ubuntu0.14.04.1'),
-                                          ('distro-info-data',
-                                           '0.18ubuntu0.2'),
-                                          ('qemu-utils',
-                                           '2.0.0+dfsg-2ubuntu1.11'),
-                                          ('unity-services',
-                                           '7.2.5+14.04.20150521.1-0ubuntu1'),
-                                         ], False, self.cachedir)
-
-        def sandbox_ver(pkg, debian=True):
-            if debian:
-                changelog = 'changelog.Debian.gz'
-            else:
-                changelog = 'changelog.gz'
-            with gzip.open(os.path.join(self.rootdir, 'usr/share/doc', pkg,
-                                        changelog)) as f:
-                return f.readline().decode().split()[1][1:-1]
-
-        self.assertEqual(obsolete, '')
-
-        # packages get installed
-        self.assertTrue(os.path.exists(os.path.join(self.rootdir,
-                                                    'usr/share/doc/oxideqt-codecs/copyright')))
-        self.assertTrue(os.path.exists(os.path.join(self.rootdir,
-                                                    'usr/share/distro-info/ubuntu.csv')))
-
-        # their versions are as expected
-        self.assertEqual(sandbox_ver('oxideqt-codecs'),
-                         '1.6.6-0ubuntu0.14.04.1')
-        self.assertEqual(sandbox_ver('oxideqt-codecs-dbg'),
-                         '1.6.6-0ubuntu0.14.04.1')
-        self.assertEqual(sandbox_ver('distro-info-data', debian=False),
-                         '0.18ubuntu0.2')
-
-        # keeps track of package versions
-        with open(os.path.join(self.rootdir, 'packages.txt')) as f:
-            pkglist = f.read().splitlines()
-        self.assertIn('oxideqt-codecs 1.6.6-0ubuntu0.14.04.1', pkglist)
-        self.assertIn('oxideqt-codecs-dbg 1.6.6-0ubuntu0.14.04.1', pkglist)
-        self.assertIn('distro-info-data 0.18ubuntu0.2', pkglist)
-        self.assertIn('qemu-utils-dbgsym 2.0.0+dfsg-2ubuntu1.11',
-                      pkglist)
-        # FIXME: doesn't work in autopkgtest
-        # self.assertIn('unity-services-dbgsym 7.2.5+14.04.20150521.1-0ubuntu1',
-        #               pkglist)
-
-        # caches packages, and their versions are as expected
-        cache = os.listdir(os.path.join(self.cachedir, 'Foonux 14.04', 'apt',
-                                        'var', 'cache', 'apt', 'archives'))
-
-        # archive and launchpad versions of packages exist in the cache, so use a list
-        cache_versions = []
-        for p in cache:
-            try:
-                (name, ver) = p.split('_')[:2]
-                cache_versions.append((name, ver))
-            except ValueError:
-                pass  # not a .deb, ignore
-        self.assertIn(('oxideqt-codecs', '1.6.6-0ubuntu0.14.04.1'), cache_versions)
-        self.assertIn(('oxideqt-codecs-dbg', '1.6.6-0ubuntu0.14.04.1'), cache_versions)
-        self.assertIn(('distro-info-data', '0.18ubuntu0.2'), cache_versions)
-        self.assertIn(('qemu-utils-dbgsym', '2.0.0+dfsg-2ubuntu1.11'), cache_versions)
-
     @unittest.skipUnless(_has_internet(), 'online test')
     def test_install_old_packages(self):
         '''sandbox will install older package versions from launchpad'''
-
-        self._setup_foonux_config()
-        obsolete = impl.install_packages(self.rootdir, self.configdir, 'Foonux 14.04',
-                                         [('oxideqt-codecs',
-                                           '1.7.8-0ubuntu0.14.04.1'),
-                                         ], False, self.cachedir)
+        release = self._setup_foonux_config(release='jammy')
+        wanted_package = "libcurl4"
+        wanted_version = "7.81.0-1"  # pre-release version
+        obsolete = impl.install_packages(self.rootdir, self.configdir, release,
+                                         [(wanted_package, wanted_version)],
+                                         False, self.cachedir)
 
         self.assertEqual(obsolete, '')
 
@@ -949,29 +867,27 @@ deb http://secondary.mirror tuxy extra
                 return f.readline().decode().split()[1][1:-1]
 
         # the version is as expected
-        self.assertEqual(sandbox_ver('oxideqt-codecs'),
-                         '1.7.8-0ubuntu0.14.04.1')
+        self.assertEqual(sandbox_ver(wanted_package), wanted_version)
 
         # keeps track of package version
         with open(os.path.join(self.rootdir, 'packages.txt')) as f:
             pkglist = f.read().splitlines()
-        self.assertIn('oxideqt-codecs 1.7.8-0ubuntu0.14.04.1', pkglist)
+        self.assertIn(f"{wanted_package} {wanted_version}", pkglist)
 
-        obsolete = impl.install_packages(self.rootdir, self.configdir, 'Foonux 14.04',
-                                         [('oxideqt-codecs',
-                                           '1.6.6-0ubuntu0.14.04.1'),
-                                         ], False, self.cachedir)
+        wanted_version = "7.74.0-1.3ubuntu3"
+        obsolete = impl.install_packages(self.rootdir, self.configdir, release,
+                                         [(wanted_package, wanted_version)],
+                                         False, self.cachedir)
 
         self.assertEqual(obsolete, '')
 
         # the old version is installed
-        self.assertEqual(sandbox_ver('oxideqt-codecs'),
-                         '1.6.6-0ubuntu0.14.04.1')
+        self.assertEqual(sandbox_ver(wanted_package), wanted_version)
 
         # the old versions is tracked
         with open(os.path.join(self.rootdir, 'packages.txt')) as f:
             pkglist = f.read().splitlines()
-        self.assertIn('oxideqt-codecs 1.6.6-0ubuntu0.14.04.1', pkglist)
+        self.assertIn(f"{wanted_package} {wanted_version}", pkglist)
 
     @unittest.skipUnless(_has_internet(), 'online test')
     def test_get_source_tree_sandbox(self):
@@ -1060,10 +976,10 @@ deb http://secondary.mirror tuxy extra
     def test_install_package_from_a_ppa(self):
         '''Install a package from a PPA.'''
         ppa = 'LP-PPA-brian-murray'
-        self._setup_foonux_config()
-        obsolete = impl.install_packages(self.rootdir, self.configdir, 'Foonux 14.04',
+        self._setup_foonux_config(release='xenial')
+        obsolete = impl.install_packages(self.rootdir, self.configdir, 'Foonux 16.04',
                                          [('apport',
-                                           '2.14.1-0ubuntu3.7~ppa4')
+                                           '2.20.1-0ubuntu2.22~ppa1')
                                          ], False, self.cachedir, origins=[ppa])
 
         self.assertEqual(obsolete, '')
@@ -1074,7 +990,25 @@ deb http://secondary.mirror tuxy extra
                 return f.readline().decode().split()[1][1:-1]
 
         self.assertEqual(sandbox_ver('apport'),
-                         '2.14.1-0ubuntu3.7~ppa4')
+                         '2.20.1-0ubuntu2.22~ppa1')
+
+    def _get_library_path(self, library_name: str, root_dir: str = "/") -> str:
+        """Find library path regardless of the architecture."""
+        libraries = glob.glob(
+            os.path.join(root_dir, "usr/lib/*", library_name)
+        )
+        self.assertEqual(
+            len(libraries), 1, f"glob for {library_name}: {libraries!r}"
+        )
+        return libraries[0]
+
+    def _ubuntu_archive_uri(self, arch=None):
+        """Return archive URI for the given architecture."""
+        if arch is None:
+            arch = impl.get_system_architecture()
+        if arch in ("amd64", "i386"):
+            return "http://archive.ubuntu.com/ubuntu"
+        return "http://ports.ubuntu.com/ubuntu-ports"
 
     def _setup_foonux_config(self, updates=False, release='trusty', ppa=False):
         '''Set up directories and configuration for install_packages()
@@ -1086,7 +1020,8 @@ deb http://secondary.mirror tuxy extra
         versions = {'trusty': '14.04',
                     'xenial': '16.04',
                     'boinic': '18.04',
-                    'cosmic': '18.10'}
+                    'cosmic': '18.10',
+                    'jammy': '22.04'}
         vers = versions[release]
         self.cachedir = os.path.join(self.workdir, 'cache')
         self.rootdir = os.path.join(self.workdir, 'root')
@@ -1095,28 +1030,26 @@ deb http://secondary.mirror tuxy extra
         os.mkdir(self.rootdir)
         os.mkdir(self.configdir)
         os.mkdir(os.path.join(self.configdir, 'Foonux %s' % vers))
-        with open(os.path.join(self.configdir, 'Foonux %s' % vers, 'sources.list'), 'w') as f:
-            f.write('deb http://archive.ubuntu.com/ubuntu/ %s main\n' % release)
-            f.write('deb-src http://archive.ubuntu.com/ubuntu/ %s main\n' % release)
-            f.write('deb http://ddebs.ubuntu.com/ %s main\n' % release)
-            if updates:
-                f.write('deb http://archive.ubuntu.com/ubuntu/ %s-updates main\n' % release)
-                f.write('deb-src http://archive.ubuntu.com/ubuntu/ %s-updates main\n' % release)
-                f.write('deb http://ddebs.ubuntu.com/ %s-updates main\n' % release)
+        self._write_source_file(
+            os.path.join(self.configdir, 'Foonux %s' % vers, 'sources.list'),
+            self._ubuntu_archive_uri(),
+            release,
+            updates,
+        )
         if ppa:
             os.mkdir(os.path.join(self.configdir, 'Foonux %s' % vers, 'sources.list.d'))
             with open(os.path.join(self.configdir, 'Foonux %s' % vers, 'sources.list.d', 'fooser-bar-ppa.list'), 'w') as f:
-                f.write('deb http://ppa.launchpad.net/fooser/bar-ppa/ubuntu %s main main/debug\n' % release)
-                f.write('deb-src http://ppa.launchpad.net/fooser/bar-ppa/ubuntu %s main\n' % release)
+                f.write(
+                    f'deb http://ppa.launchpad.net/fooser/bar-ppa/ubuntu {release} main main/debug\n'
+                    f'deb-src http://ppa.launchpad.net/fooser/bar-ppa/ubuntu {release} main\n'
+                )
         os.mkdir(os.path.join(self.configdir, 'Foonux %s' % vers, 'armhf'))
-        with open(os.path.join(self.configdir, 'Foonux %s' % vers, 'armhf', 'sources.list'), 'w') as f:
-            f.write('deb http://ports.ubuntu.com/ %s main\n' % release)
-            f.write('deb-src http://ports.ubuntu.com/ %s main\n' % release)
-            f.write('deb http://ddebs.ubuntu.com/ %s main\n' % release)
-            if updates:
-                f.write('deb http://ports.ubuntu.com/ %s-updates main\n' % release)
-                f.write('deb-src http://ports.ubuntu.com/ %s-updates main\n' % release)
-                f.write('deb http://ddebs.ubuntu.com/ %s-updates main\n' % release)
+        self._write_source_file(
+            os.path.join(self.configdir, 'Foonux %s' % vers, 'armhf', 'sources.list'),
+            self._ubuntu_archive_uri("armhf"),
+            release,
+            updates,
+        )
         with open(os.path.join(self.configdir, 'Foonux %s' % vers, 'codename'), 'w') as f:
             f.write('%s' % release)
 
@@ -1133,6 +1066,22 @@ deb http://secondary.mirror tuxy extra
         # that looks for trusted.gpg.d relative to sources.list.
         keyring_arch_dir = os.path.join(self.configdir, 'Foonux %s' % vers, 'armhf', 'trusted.gpg.d')
         os.symlink("../trusted.gpg.d", keyring_arch_dir)
+        return f'Foonux {vers}'
+
+    def _write_source_file(self, sources_filename: str, uri: str, release: str, updates: bool) -> None:
+        '''Write sources.list file.'''
+        with open(sources_filename, 'w') as sources_file:
+            sources_file.write(
+                f'deb {uri} {release} main\n'
+                f'deb-src {uri} {release} main\n'
+                f'deb http://ddebs.ubuntu.com/ {release} main\n'
+            )
+            if updates:
+                sources_file.write(
+                    f'deb {uri} {release}-updates main\n'
+                    f'deb-src {uri} {release}-updates main\n'
+                    f'deb http://ddebs.ubuntu.com/ {release}-updates main\n'
+                )
 
     def assert_elf_arch(self, path, expected):
         '''Assert that an ELF file is for an expected machine type.
@@ -1140,9 +1089,12 @@ deb http://secondary.mirror tuxy extra
         Expected is a Debian-style architecture (i386, amd64, armhf)
         '''
         archmap = {
-            'i386': '80386',
-            'amd64': 'X86-64',
-            'armhf': 'ARM',
+            "amd64": "X86-64",
+            "arm64": "AArch64",
+            "armhf": "ARM",
+            "i386": "80386",
+            "ppc64el": "PowerPC64",
+            "s390x": "IBM S/390",
         }
 
         # get ELF machine type
