@@ -8,10 +8,9 @@
 # the full text of the license.
 
 import tempfile, shutil, os, subprocess, signal, time, stat, sys
-import resource, errno, grp, unittest, socket, array, pwd
+import resource, errno, grp, unittest, socket, array
 import apport.fileutils
 
-test_executable = '/usr/bin/yes'
 test_package = 'coreutils'
 test_source = 'coreutils'
 
@@ -33,16 +32,33 @@ if orig_home is not None:
     os.environ['HOME'] = orig_home
 
 # did we enable suid_dumpable?
-suid_dumpable = False
+suid_dumpable = 0
 try:
     with open('/proc/sys/fs/suid_dumpable') as f:
-        if f.read().strip() != '0':
-            suid_dumpable = True
+        suid_dumpable = int(f.read().strip())
 except IOError:
     pass
 
 
+def pidof(program):
+    """Find the process ID of a running program.
+
+    This function wraps the pidof command and returns a set of
+    process IDs.
+    """
+    try:
+        stdout = subprocess.check_output(["/bin/pidof", "-nx", program])
+    except subprocess.CalledProcessError as error:
+        if error.returncode == 1:
+            return set()
+        raise  # pragma: no cover
+    return set(int(pid) for pid in stdout.decode().split())
+
+
 class T(unittest.TestCase):
+    TEST_EXECUTABLE = os.path.realpath("/bin/sleep")
+    TEST_ARGS = ["86400"]
+
     def setUp(self):
         # use local report dir
         self.report_dir = tempfile.mkdtemp()
@@ -63,7 +79,7 @@ class T(unittest.TestCase):
         # expected report name for test executable report
         self.test_report = os.path.join(
             apport.fileutils.report_dir, '%s.%i.crash' %
-            (test_executable.replace('/', '_'), os.getuid()))
+            (self.TEST_EXECUTABLE.replace('/', '_'), os.getuid()))
 
     def tearDown(self):
         shutil.rmtree(self.report_dir)
@@ -127,8 +143,11 @@ class T(unittest.TestCase):
             pr.load(f)
         self.assertTrue(set(required_fields).issubset(set(pr.keys())),
                         'report has required fields')
-        self.assertEqual(pr['ExecutablePath'], test_executable)
-        self.assertEqual(pr['ProcCmdline'], test_executable)
+        self.assertEqual(pr['ExecutablePath'], self.TEST_EXECUTABLE)
+        self.assertEqual(
+            pr['ProcCmdline'],
+            " ".join([self.TEST_EXECUTABLE] + self.TEST_ARGS),
+        )
         self.assertEqual(pr['Signal'], '%i' % signal.SIGSEGV)
 
         # check safe environment subset
@@ -138,13 +157,15 @@ class T(unittest.TestCase):
                         'LC_TELEPHONE', 'LC_MEASUREMENT', 'LC_IDENTIFICATION',
                         'LOCPATH', 'TERM', 'XDG_RUNTIME_DIR', 'LD_PRELOAD']
 
-        for l in pr['ProcEnviron'].splitlines():
-            (k, v) = l.split('=', 1)
+        for line in pr['ProcEnviron'].splitlines():
+            (k, v) = line.split('=', 1)
             self.assertTrue(k in allowed_vars,
                             'report contains sensitive environment variable %s' % k)
 
         # UserGroups only has system groups
         for g in pr['UserGroups'].split():
+            if g == 'N/A':
+                continue
             self.assertLess(grp.getgrnam(g).gr_gid, 500)
 
         self.assertFalse('root' in pr['UserGroups'],
@@ -155,7 +176,7 @@ class T(unittest.TestCase):
         '''only one apport instance is ran at a time'''
 
         test_proc = self.create_test_process()
-        test_proc2 = self.create_test_process(False, '/bin/dd')
+        test_proc2 = self.create_test_process(False, '/bin/dd', args=[])
         try:
             app = subprocess.Popen([apport_path, str(test_proc), '42', '0', '1'],
                                    stdin=subprocess.PIPE, stderr=subprocess.PIPE)
@@ -192,40 +213,12 @@ class T(unittest.TestCase):
             os.kill(test_proc2, 9)
             os.waitpid(test_proc2, 0)
 
-    def test_lock_symlink(self):
-        '''existing .lock file as dangling symlink does not create the file
-
-        This would be a vulnerability, as users could overwrite system files.
-        '''
-        # prepare a symlink trap
-        lockpath = os.path.join(self.report_dir, '.lock')
-        trappath = os.path.join(self.report_dir, '0wned')
-        os.symlink(trappath, lockpath)
-
-        # now call apport
-        test_proc = self.create_test_process()
-
-        try:
-            app = subprocess.Popen([apport_path, str(test_proc), '42', '0', '1'],
-                                   stdin=subprocess.PIPE, stderr=subprocess.PIPE)
-            app.stdin.write(b'boo')
-            app.stdin.close()
-
-            self.assertNotEqual(app.wait(), 0, app.stderr.read())
-            app.stderr.close()
-        finally:
-            os.kill(test_proc, 9)
-            os.waitpid(test_proc, 0)
-
-        self.assertEqual(self.get_temp_all_reports(), [])
-        self.assertFalse(os.path.exists(trappath))
-
     def test_unpackaged_binary(self):
         '''unpackaged binaries do not create a report'''
 
         local_exe = os.path.join(self.workdir, 'mybin')
         with open(local_exe, 'wb') as dest:
-            with open(test_executable, 'rb') as src:
+            with open(self.TEST_EXECUTABLE, 'rb') as src:
                 dest.write(src.read())
         os.chmod(local_exe, 0o755)
         self.do_crash(command=local_exe)
@@ -238,14 +231,14 @@ class T(unittest.TestCase):
         with open(local_exe, 'w') as f:
             f.write('#!/bin/sh\nkill -SEGV $$')
         os.chmod(local_exe, 0o755)
-        self.do_crash(command=local_exe)
+        self.do_crash(command=local_exe, args=[])
 
         # absolute path
         self.assertEqual(apport.fileutils.get_all_reports(), [])
 
         # relative path
         os.chdir(self.workdir)
-        self.do_crash(command='./myscript')
+        self.do_crash(command='./myscript', args=[])
         self.assertEqual(apport.fileutils.get_all_reports(), [])
 
     def test_ignore_sigquit(self):
@@ -261,7 +254,7 @@ class T(unittest.TestCase):
         with open(local_exe, 'w') as f:
             f.write('#!/usr/bin/perl\nsystem("mv $0 $0.exe");\nsystem("ln -sf /etc/shadow $0");\n$0="..$0";\nsleep(10);\n')
         os.chmod(local_exe, 0o755)
-        self.do_crash(check_running=False, command=local_exe, sleep=2)
+        self.do_crash(check_running=False, command=local_exe, args=[], sleep=2)
 
         leak = os.path.join(apport.fileutils.report_dir, '_usr_bin_perl.%i.crash' %
                             (os.getuid()))
@@ -315,7 +308,7 @@ class T(unittest.TestCase):
         # regards as likely packaged
         (fd, myexe) = tempfile.mkstemp(dir='/var/tmp')
         self.addCleanup(os.unlink, myexe)
-        with open(test_executable, 'rb') as f:
+        with open(self.TEST_EXECUTABLE, 'rb') as f:
             os.write(fd, f.read())
         os.close(fd)
         os.chmod(myexe, 0o111)
@@ -324,7 +317,8 @@ class T(unittest.TestCase):
         resource.setrlimit(resource.RLIMIT_CORE, (-1, -1))
 
         if suid_dumpable:
-            self.do_crash(True, command=myexe, expect_corefile=False)
+            expected_owner = {1: os.getuid(), 2: 0}[suid_dumpable]
+            self.do_crash(True, command=myexe, expect_corefile=True, expect_corefile_owner=expected_owner)
 
             # check crash report
             reports = apport.fileutils.get_new_system_reports()
@@ -334,8 +328,8 @@ class T(unittest.TestCase):
             # FIXME: we would like to clean up this, but don't have privileges for that
             # os.unlink(report)
             self.assertEqual(stat.S_IMODE(st.st_mode), 0o640, 'report has correct permissions')
-            # this must be owned by root as it is an unreadable binary
-            self.assertEqual(st.st_uid, 0, 'report has correct owner')
+            # this must be owned by root for suid_dumpable == 2 as it is an unreadable binary
+            self.assertEqual(st.st_uid, expected_owner, 'report has correct owner')
 
             # no user reports
             self.assertEqual(apport.fileutils.get_all_reports(), [])
@@ -366,8 +360,9 @@ class T(unittest.TestCase):
             self.do_crash(expect_corefile=True)
             apport.fileutils.delete_report(self.test_report)
 
-        # for SIGQUIT we only expect core files, no report
-        resource.setrlimit(resource.RLIMIT_CORE, (1000000, -1))
+    def test_core_dump_packaged_sigquit(self):
+        '''packaged executables create core files, no report for SIGQUIT'''
+        resource.setrlimit(resource.RLIMIT_CORE, (-1, -1))
         self.do_crash(expect_corefile=True, sig=signal.SIGQUIT)
         self.assertEqual(apport.fileutils.get_all_reports(), [])
 
@@ -376,7 +371,7 @@ class T(unittest.TestCase):
 
         local_exe = os.path.join(self.workdir, 'mybin')
         with open(local_exe, 'wb') as dest:
-            with open(test_executable, 'rb') as src:
+            with open(self.TEST_EXECUTABLE, 'rb') as src:
                 dest.write(src.read())
         os.chmod(local_exe, 0o755)
 
@@ -408,6 +403,11 @@ CoreDump: base64
         # crash our test process and let it write a core file
         resource.setrlimit(resource.RLIMIT_CORE, (-1, -1))
         pid = self.create_test_process()
+
+        # get the name of the core file
+        (core_name, core_path) = apport.fileutils.get_core_path(pid,
+                                                                self.TEST_EXECUTABLE)
+
         os.kill(pid, signal.SIGSEGV)
 
         # replace report with the crafted one above as soon as it exists and
@@ -426,13 +426,11 @@ CoreDump: base64
         os.sync()
 
         # verify that we get the original core, not the injected one
-        with open('core', 'rb') as f:
+        with open(core_path, 'rb') as f:
             core = f.read()
         self.assertNotIn(b'pwned', core)
         self.assertGreater(len(core), 10000)
 
-        os.unlink('core')
-
     def test_limit_size(self):
         '''core dumps are capped on available memory size'''
 
@@ -482,12 +480,15 @@ CoreDump: base64
         os.unlink(reports[0])
 
         self.assertEqual(pr['Signal'], '42')
-        self.assertEqual(pr['ExecutablePath'], test_executable)
+        self.assertEqual(pr['ExecutablePath'], self.TEST_EXECUTABLE)
         self.assertFalse('CoreDump' in pr)
         # FIXME: sometimes this is empty!?
         if err:
             self.assertRegex(
-                err, b'core dump exceeded.*dropped from .*yes\\..*\\.crash')
+                err.decode(),
+                f"core dump exceeded.*dropped from .*"
+                f"{os.path.basename(self.TEST_EXECUTABLE)}\\..*\\.crash",
+            )
 
     def test_ignore(self):
         '''ignoring executables'''
@@ -514,7 +515,7 @@ CoreDump: base64
         # likely packaged
         (fd, myexe) = tempfile.mkstemp(dir='/var/tmp')
         self.addCleanup(os.unlink, myexe)
-        with open(test_executable, 'rb') as f:
+        with open(self.TEST_EXECUTABLE, 'rb') as f:
             os.write(fd, f.read())
         os.close(fd)
         os.chmod(myexe, 0o755)
@@ -580,7 +581,7 @@ CoreDump: base64
         os.unlink(reports[0])
 
         self.assertEqual(pr['Signal'], '42')
-        self.assertEqual(pr['ExecutablePath'], test_executable)
+        self.assertEqual(pr['ExecutablePath'], self.TEST_EXECUTABLE)
         self.assertEqual(pr['CoreDump'], b'hel\x01lo')
 
     def test_logging_stderr(self):
@@ -615,7 +616,7 @@ CoreDump: base64
         os.unlink(reports[0])
 
         self.assertEqual(pr['Signal'], '42')
-        self.assertEqual(pr['ExecutablePath'], test_executable)
+        self.assertEqual(pr['ExecutablePath'], self.TEST_EXECUTABLE)
         self.assertEqual(pr['CoreDump'], b'hel\x01lo')
 
     @unittest.skipIf(os.geteuid() != 0, 'this test needs to be run as root')
@@ -626,7 +627,7 @@ CoreDump: base64
         # regards as likely packaged
         (fd, myexe) = tempfile.mkstemp(dir='/var/tmp')
         self.addCleanup(os.unlink, myexe)
-        with open(test_executable, 'rb') as f:
+        with open(self.TEST_EXECUTABLE, 'rb') as f:
             os.write(fd, f.read())
         os.close(fd)
         os.chmod(myexe, 0o4755)
@@ -635,8 +636,10 @@ CoreDump: base64
         resource.setrlimit(resource.RLIMIT_CORE, (-1, -1))
 
         if suid_dumpable:
-            # if a user can crash a suid root binary, it should not create core files
-            self.do_crash(command=myexe, uid=8)
+            expected_owner = {1: 8, 2: 0}[suid_dumpable]
+            # If a user can crash a suid root binary, the core file should
+            # only be readable by root (for suid_dumpable == 2).
+            self.do_crash(command=myexe, uid=8, expect_corefile=True, expect_corefile_owner=expected_owner)
 
             # check crash report
             reports = apport.fileutils.get_all_reports()
@@ -645,13 +648,38 @@ CoreDump: base64
             st = os.stat(report)
             os.unlink(report)
             self.assertEqual(stat.S_IMODE(st.st_mode), 0o640, 'report has correct permissions')
-            # this must be owned by root as it is a setuid binary
-            self.assertEqual(st.st_uid, 0, 'report has correct owner')
+            # this must be owned by root for suid_dumpable == 2 as it is a setuid binary
+            self.assertEqual(st.st_uid, expected_owner, 'report has correct owner')
         else:
             # no cores/dump if suid_dumpable == 0
             self.do_crash(False, command=myexe, expect_corefile=False, uid=8)
             self.assertEqual(apport.fileutils.get_all_reports(), [])
 
+    @unittest.skipIf(os.geteuid() != 0, 'this test needs to be run as root')
+    def test_crash_system_slice(self):
+        '''report generation for a protected process running in the system slice'''
+
+        self.assertEqual(pidof(self.TEST_EXECUTABLE), set(), 'no running test executable processes')
+        self.create_test_process(command='/usr/bin/systemd-run',
+                                 check_running=False,
+                                 args=['-t', '-q', '--slice=system.slice',
+                                       '-p', 'ProtectSystem=true',
+                                       self.TEST_EXECUTABLE] + self.TEST_ARGS)
+        pids = pidof(self.TEST_EXECUTABLE)
+        self.assertEqual(len(pids), 1)
+        os.kill(pids.pop(), signal.SIGSEGV)
+
+        self.wait_for_no_instance_running(self.TEST_EXECUTABLE)
+        self.wait_for_apport_to_finish()
+
+        # check crash report
+        reports = apport.fileutils.get_all_reports()
+        self.assertEqual(len(reports), 1)
+        self.assertEqual(
+            reports[0],
+            f"/var/crash/{self.TEST_EXECUTABLE.replace('/', '_')}.0.crash",
+        )
+
     @unittest.skipUnless(os.path.exists('/bin/ping'), 'this test needs /bin/ping')
     @unittest.skipIf(os.geteuid() != 0, 'this test needs to be run as root')
     def test_crash_setuid_drop(self):
@@ -661,8 +689,16 @@ CoreDump: base64
         resource.setrlimit(resource.RLIMIT_CORE, (-1, -1))
 
         if suid_dumpable:
-            # if a user can crash a suid root binary, it should not create core files
-            self.do_crash(command='/bin/ping', args=['127.0.0.1'], uid=8)
+            expected_owner = {1: 8, 2: 0}[suid_dumpable]
+            # If a user can crash a suid root binary, the core file should
+            # only be readable by root (for suid_dumpable == 2).
+            self.do_crash(
+                command=os.path.realpath('/bin/ping'),
+                args=['127.0.0.1'],
+                expect_corefile=True,
+                expect_corefile_owner=expected_owner,
+                uid=8,
+            )
 
             # check crash report
             reports = apport.fileutils.get_all_reports()
@@ -671,45 +707,8 @@ CoreDump: base64
             st = os.stat(report)
             os.unlink(report)
             self.assertEqual(stat.S_IMODE(st.st_mode), 0o640, 'report has correct permissions')
-            # this must be owned by root as it is a setuid binary
-            self.assertEqual(st.st_uid, 0, 'report has correct owner')
-        else:
-            # no cores/dump if suid_dumpable == 0
-            self.do_crash(False, command='/bin/ping', args=['127.0.0.1'],
-                          uid=8)
-            self.assertEqual(apport.fileutils.get_all_reports(), [])
-
-    @unittest.skipUnless(os.path.exists('/bin/ping'), 'this test needs /bin/ping')
-    @unittest.skipIf(os.geteuid() != 0, 'this test needs to be run as root')
-    def test_crash_setuid_drop_and_kill(self):
-        '''process started by root as another user, killed by that user no core'''
-        # override expected report file name
-        self.test_report = os.path.join(
-            apport.fileutils.report_dir, '%s.%i.crash' %
-            ('_usr_bin_crontab', os.getuid()))
-        # edit crontab as user "mail"
-        resource.setrlimit(resource.RLIMIT_CORE, (-1, -1))
-
-        if suid_dumpable:
-            user = pwd.getpwuid(8)
-            # if a user can crash a suid root binary, it should not create core files
-            orig_editor = os.getenv('EDITOR')
-            os.environ['EDITOR'] = '/usr/bin/yes'
-            self.do_crash(command='/usr/bin/crontab', args=['-e', '-u', user[0]],
-                          expect_corefile=False, core_location='/var/spool/cron/',
-                          killer_id=8)
-            if orig_editor is not None:
-                os.environ['EDITOR'] = orig_editor
-
-            # check crash report
-            reports = apport.fileutils.get_all_reports()
-            self.assertEqual(len(reports), 1)
-            report = reports[0]
-            st = os.stat(report)
-            os.unlink(report)
-            self.assertEqual(stat.S_IMODE(st.st_mode), 0o640, 'report has correct permissions')
-            # this must be owned by root as it is a setuid binary
-            self.assertEqual(st.st_uid, 0, 'report has correct owner')
+            # this must be owned by root for suid_dumpable == 2 as it is a setuid binary
+            self.assertEqual(st.st_uid, expected_owner, 'report has correct owner')
         else:
             # no cores/dump if suid_dumpable == 0
             self.do_crash(False, command='/bin/ping', args=['127.0.0.1'],
@@ -724,7 +723,7 @@ CoreDump: base64
         # regards as not packaged
         (fd, myexe) = tempfile.mkstemp(dir='/tmp')
         self.addCleanup(os.unlink, myexe)
-        with open(test_executable, 'rb') as f:
+        with open(self.TEST_EXECUTABLE, 'rb') as f:
             os.write(fd, f.read())
         os.close(fd)
         os.chmod(myexe, 0o4755)
@@ -733,8 +732,15 @@ CoreDump: base64
         resource.setrlimit(resource.RLIMIT_CORE, (-1, -1))
 
         if suid_dumpable:
-            # if a user can crash a suid root binary, it should not create core files
-            self.do_crash(command=myexe, expect_corefile=False, uid=8)
+            expected_owner = {1: 8, 2: 0}[suid_dumpable]
+            # If a user can crash a suid root binary, the core file should
+            # only be readable by root (for suid_dumpable == 2).
+            self.do_crash(
+                command=myexe,
+                expect_corefile=True,
+                expect_corefile_owner=expected_owner,
+                uid=8,
+            )
         else:
             # no cores/dump if suid_dumpable == 0
             self.do_crash(False, command=myexe, expect_corefile=False, uid=8)
@@ -750,7 +756,7 @@ CoreDump: base64
         # regards as likely packaged
         (fd, myexe) = tempfile.mkstemp(dir='/var/tmp')
         self.addCleanup(os.unlink, myexe)
-        with open(test_executable, 'rb') as f:
+        with open(self.TEST_EXECUTABLE, 'rb') as f:
             os.write(fd, f.read())
         os.close(fd)
         os.chmod(myexe, 0o4755)
@@ -761,8 +767,13 @@ CoreDump: base64
         resource.setrlimit(resource.RLIMIT_CORE, (-1, -1))
 
         if suid_dumpable:
-            # we expect a report, but no core file
-            self.do_crash(command=myexe, expect_corefile=False, uid=8)
+            expected_owner = {1: 8, 2: 0}[suid_dumpable]
+            self.do_crash(
+                command=myexe,
+                expect_corefile=True,
+                expect_corefile_owner=expected_owner,
+                uid=8,
+            )
 
             # check crash report
             reports = apport.fileutils.get_all_reports()
@@ -771,8 +782,8 @@ CoreDump: base64
             st = os.stat(report)
             os.unlink(report)
             self.assertEqual(stat.S_IMODE(st.st_mode), 0o640, 'report has correct permissions')
-            # this must be owned by root as it is a setuid binary
-            self.assertEqual(st.st_uid, 0, 'report has correct owner')
+            # this must be owned by root for suid_dumpable == 2 as it is a setuid binary
+            self.assertEqual(st.st_uid, expected_owner, 'report has correct owner')
         else:
             # no core/report if suid_dumpable == 0
             self.do_crash(False, command=myexe, expect_corefile=False, uid=8)
@@ -830,7 +841,7 @@ CoreDump: base64
             pr.load(f)
         os.unlink(reports[0])
         self.assertEqual(pr['Signal'], '11')
-        self.assertEqual(pr['ExecutablePath'], test_executable)
+        self.assertEqual(pr['ExecutablePath'], self.TEST_EXECUTABLE)
         self.assertEqual(pr['CoreDump'], b'hel\x01lo')
 
         # should not create report on the host
@@ -840,13 +851,17 @@ CoreDump: base64
     # Helper methods
     #
 
-    @classmethod
-    def create_test_process(klass, check_running=True, command=test_executable,
-                            uid=None, args=[]):
-        '''Spawn test_executable.
+    def create_test_process(self, check_running=True, command=None,
+                            uid=None, args=None):
+        '''Spawn test executable.
 
         Wait until it is fully running, and return its PID.
         '''
+        if command is None:
+            command = self.TEST_EXECUTABLE
+        if args is None:
+            args = self.TEST_ARGS
+
         assert os.access(command, os.X_OK), command + ' is not executable'
         if check_running:
             assert subprocess.call(['pidof', command]) == 1, 'no running test executable processes'
@@ -876,13 +891,12 @@ CoreDump: base64
 
     def do_crash(self, expect_coredump=True, expect_corefile=False,
                  sig=signal.SIGSEGV, check_running=True, sleep=0,
-                 command=test_executable, uid=None,
+                 command=None, uid=None,
                  expect_corefile_owner=None,
-                 core_location=None,
-                 killer_id=False, args=[]):
+                 args=None):
         '''Generate a test crash.
 
-        This runs command (by default test_executable) in cwd, lets it crash,
+        This runs command (by default TEST_EXECUTABLE) in cwd, lets it crash,
         and checks that it exits with the expected return code, leaving a core
         file behind if expect_corefile is set, and generating a crash report if
         expect_coredump is set.
@@ -890,25 +904,20 @@ CoreDump: base64
         If check_running is set (default), this will abort if test_process is
         already running.
         '''
+        if command is None:
+            command = self.TEST_EXECUTABLE
+        if args is None:
+            args = self.TEST_ARGS
         self.assertFalse(os.path.exists('core'), '%s/core already exists, please clean up first' % os.getcwd())
         pid = self.create_test_process(check_running, command, uid=uid, args=args)
+
+        (core_name, core_path) = apport.fileutils.get_core_path(pid,
+                                                                command,
+                                                                uid)
+
         if sleep > 0:
             time.sleep(sleep)
-        if killer_id:
-            user = pwd.getpwuid(killer_id)
-            # testing different editors via VISUAL= didn't help
-            kill = subprocess.Popen(['sudo', '-s', '/bin/bash', '-c',
-                                     "/bin/kill -s %i %s" % (sig, pid),
-                                     '-u', user[0]])  # 'mail'])
-            kill.communicate()
-            # need to clean up system state
-            if command == '/usr/bin/crontab':
-                os.system('stty sane')
-            if kill.returncode != 0:
-                self.fail("Couldn't kill process %s as user %s." %
-                          (pid, user[0]))
-        else:
-            os.kill(pid, sig)
+        os.kill(pid, sig)
         # wait max 5 seconds for the process to die
         timeout = 50
         while timeout >= 0:
@@ -921,40 +930,23 @@ CoreDump: base64
             os.kill(pid, signal.SIGKILL)
             os.waitpid(pid, 0)
             self.fail('test process does not die on signal %i' % sig)
-        if command == '/usr/bin/crontab':
-            subprocess.Popen(['sudo', '-s', '/bin/bash', '-c',
-                              "/usr/bin/pkill -9 -f crontab",
-                              '-u', 'mail'])
         self.assertFalse(os.WIFEXITED(result), 'test process did not exit normally')
         self.assertTrue(os.WIFSIGNALED(result), 'test process died due to signal')
         self.assertEqual(os.WCOREDUMP(result), expect_coredump)
         self.assertEqual(os.WSTOPSIG(result), 0, 'test process was not signaled to stop')
         self.assertEqual(os.WTERMSIG(result), sig, 'test process died due to proper signal')
 
-        # wait max 10 seconds for apport to finish
-        timeout = 50
-        while timeout >= 0:
-            pidof = subprocess.Popen(['pidof', '-x', 'apport'], stdout=subprocess.PIPE)
-            pidof.communicate()
-            if pidof.returncode != 0:
-                break
-            time.sleep(0.2)
-            timeout -= 1
-        self.assertGreater(timeout, 0)
+        self.wait_for_apport_to_finish()
         if check_running:
             self.assertEqual(subprocess.call(['pidof', command]), 1,
                              'no running test executable processes')
 
-        core_path = '%s/' % os.getcwd()
-        if core_location:
-            core_path = '%s/' % core_location
-        core_path += 'core'
         if expect_corefile:
             self.assertTrue(os.path.exists(core_path), 'leaves wanted core file')
             try:
                 # check core file permissions
                 st = os.stat(core_path)
-                self.assertEqual(stat.S_IMODE(st.st_mode), 0o600, 'core file has correct permissions')
+                self.assertEqual(stat.S_IMODE(st.st_mode), 0o400, 'core file has correct permissions')
                 if expect_corefile_owner is not None:
                     self.assertEqual(st.st_uid, expect_corefile_owner, 'core file has correct owner')
 
@@ -1001,6 +993,18 @@ CoreDump: base64
         self.assertTrue('\n#2' in r.get('Stacktrace', ''),
                         r.get('Stacktrace', 'no Stacktrace field'))
 
+    def wait_for_apport_to_finish(self, timeout_sec=10.0):
+        self.wait_for_no_instance_running('apport', timeout_sec)
+
+    def wait_for_no_instance_running(self, program, timeout_sec=10.0):
+        while timeout_sec > 0:
+            if not pidof(program):
+                break
+            time.sleep(0.2)
+            timeout_sec -= 0.2
+        else:
+            self.fail(f"Timeout exceeded, but {program} is still running.")
+
 
 #
 # main
